/**********************************************************************
 *
 * queryinfo.h -- 
 * Copyright (C) 1999  The New Zealand Digital Library Project
 *
 * PUT COPYRIGHT NOTICE HERE
 *
 * $Id: queryinfo.h 398 1999-07-16 03:42:23Z sjboddie $
 *
 *********************************************************************/


#ifndef QUERYINFO_H
#define QUERYINFO_H


#include "gsdlconf.h"
#include "text_t.h"
#include "comtypes.h"

#if defined(GSDL_USE_OBJECTSPACE)
#  include <ospace\std\vector>
#  include <ospace\std\algorithm>
#elif defined(GSDL_USE_STL_H)
#  include <vector.h>
#  if defined(GSDL_USE_ALGO_H)
#    include <algo.h>
#  else
#    include <algorithm.h>
#  endif
#else
#  include <vector>
#  include <algorithm>
#endif

// query parameters

struct queryparamclass {
  text_t combinequery;
  text_t collection;

  // search_index = index+subcollection+language
  text_t index;
  text_t subcollection;
  text_t language;
  
  text_t querystring;
  int search_type; // 0 = boolean, 1 = ranked
  int casefolding;
  int stemming;
  int maxdocs;

  queryparamclass ();
  void clear ();
  queryparamclass &operator=(const queryparamclass &q);
};

bool operator==(const queryparamclass &x, const queryparamclass &y);
bool operator!=(const queryparamclass &x, const queryparamclass &y);

// stream output for debugging purposes
ostream &operator<< (ostream &outs, queryparamclass &q);



// term frequencies

struct termfreqclass {
  text_t termstr;
  text_t termstemstr;
  text_tset utf8equivterms; // kept as utf8 string for fast matching
  unsigned int termfreq;

  termfreqclass ();
  void clear();
  termfreqclass &operator=(const termfreqclass &t);
};

typedef vector<termfreqclass> termfreqclassarray;

bool operator==(const termfreqclass &x, const termfreqclass &y);
bool operator!=(const termfreqclass &x, const termfreqclass &y);
bool operator<(const termfreqclass &x, const termfreqclass &y);
bool operator>(const termfreqclass &x, const termfreqclass &y);

// stream output for debugging purposes
ostream &operator<< (ostream &outs, termfreqclass &q);



// one query result

struct docresultclass {
  int docnum;
  float docweight;
  unsigned int num_query_terms_matched;  // not available on all versions of mg
  int num_phrase_match;                  // not available on all versions of mg

  docresultclass();
  void clear ();
  
  // merges two result classes relating to a single docnum
  docresultclass &combine(const docresultclass &d);

  docresultclass &operator=(const docresultclass &d);
};

// stream output for debugging purposes
ostream &operator<< (ostream &outs, docresultclass &a);


struct ltint {
  bool operator()(const int &t1, const int &t2) const
  { return t1 < t2; }
};

typedef map<int, docresultclass, ltint> docresultmap;



// many document results

struct docresultsclass {
  docresultmap docset;
  vector<int> docorder;

  docresultsclass ();
  void clear ();
  void docnum_order();

  void combine_and (const docresultsclass &d);
  void combine_or (const docresultsclass &d);
  void combine_not (const docresultsclass &d);
  
  docresultsclass &operator=(const docresultsclass &d);
};




// query results

struct queryresultsclass {
  queryresultsclass () {clear();}

  int docs_matched; // not available on all versions of mg
  isapprox is_approx;
  //  bool is_approx;   // not available on all versions of mg

  bool postprocessed; // whether this record has been post-processed
  
  docresultsclass docs;
  termfreqclassarray orgterms; // terms before they are sorted and uniqued
  termfreqclassarray terms;
  text_tset termvariants;
  
  void clear ();
  queryresultsclass &operator=(const queryresultsclass &q);
  
  void sortuniqqueryterms();
};

// stream output for debugging purposes
ostream &operator<< (ostream &outs, queryresultsclass &q);


#endif
