/**********************************************************************
 *
 * pageaction.cpp -- 
 * Copyright (C) 1999  The New Zealand Digital Library Project
 *
 * PUT COPYRIGHT NOTICE HERE
 *
 * $Id: pageaction.cpp 430 1999-08-03 03:29:47Z sjboddie $
 *
 *********************************************************************/

/*
   $Log$
   Revision 1.9  1999/08/03 03:29:47  sjboddie
   added ability to set receptionist from collect.cfg

   Revision 1.8  1999/07/30 02:24:44  sjboddie
   added collectinfo argument to some functions

   Revision 1.7  1999/06/24 05:12:24  sjboddie
   lots of small changes

   Revision 1.6  1999/06/10 00:39:16  sjboddie
   navigation bar is no longer written out for every page (it should
   be included in the _content_ macro of pages wanting to display it).

   Revision 1.5  1999/06/08 04:29:35  sjboddie
   added argsinfo to the call to check_cgiargs to make it easy to set
   args to their default if they're found to be screwed up

   Revision 1.4  1999/02/28 20:00:14  rjmcnab


   Fixed a few things.

   Revision 1.3  1999/02/25 21:58:58  rjmcnab

   Merged sources.

   Revision 1.2  1999/02/21 22:33:54  rjmcnab

   Lots of stuff :-)

   Revision 1.1  1999/02/12 02:40:17  sjboddie

   Added page action

 */


#include "pageaction.h"
#include "receptionist.h"
#include <time.h>

pageaction::pageaction () {

  recpt = NULL;

  // this action uses cgi variables "a" and "p"
  cgiarginfo arg_ainfo;
  arg_ainfo.shortname = "a";
  arg_ainfo.longname = "action";
  arg_ainfo.multiplechar = true;
  arg_ainfo.defaultstatus = cgiarginfo::weak;
  arg_ainfo.argdefault = "p";
  arg_ainfo.savedarginfo = cgiarginfo::must;
  argsinfo.addarginfo (NULL, arg_ainfo);

  arg_ainfo.shortname = "p";
  arg_ainfo.longname = "page";
  arg_ainfo.multiplechar = true;
  arg_ainfo.defaultstatus = cgiarginfo::weak;
  arg_ainfo.argdefault = "about";
  arg_ainfo.savedarginfo = cgiarginfo::can;
  argsinfo.addarginfo (NULL, arg_ainfo);
}

pageaction::~pageaction () {
}

bool pageaction::check_cgiargs (cgiargsinfoclass &/*argsinfo*/, cgiargsclass &/*args*/, 
				ostream &/*logout*/) {
  // don't want to check anything yet.
  return true;
}

void pageaction::get_cgihead_info (cgiargsclass &/*args*/, response_t &response,
				   text_t &response_data, ostream &/*logout*/) {
  response = content;
  response_data = "text/html";
}


// define all the macros which might be used by other actions
// to produce pages. 
void pageaction::define_external_macros (const ColInfoResponse_t &/*collectinfo*/, displayclass &/*disp*/, 
					 cgiargsclass &/*args*/, recptproto */*collectproto*/, 
					 ostream &/*logout*/) {
}


void pageaction::define_internal_macros (const ColInfoResponse_t &/*collectinfo*/, displayclass &disp, 
					 cgiargsclass &args, recptproto */*collectproto*/, 
					 ostream &logout) {

  // define_internal_macros sets the following macros:

  // _homeextra_   this is the list of available collections and collection info
  //               to be displayed on the home page

  if (args["p"] == "home") {
    // make sure we know about a receptionist
    if (recpt == NULL) {
      logout << "The page action does not contain information\n"
	     << "about any receptionists. The method set_receptionist\n"
	     <<	"was probably not called from the module which instantiated\n"
	     <<	"this page action.\n";
      return;
    }

    unsigned long current_time = time(NULL);
    text_t homeextra;

    recptprotolistclass *rprotolist = recpt->get_recptprotolist_ptr ();
    if (rprotolist == NULL) return;

    recptprotolistclass::iterator rprotolist_here = rprotolist->begin();
    recptprotolistclass::iterator rprotolist_end = rprotolist->end();
    while (rprotolist_here != rprotolist_end) {
      if ((*rprotolist_here).p != NULL) {
	text_tarray collist;
	comerror_t err;
	(*rprotolist_here).p->get_collection_list (collist, err, logout);
	if (err == noError) {
	  text_tarray::iterator collist_here = collist.begin();
	  text_tarray::iterator collist_end = collist.end();

	  homeextra += "<dl>\n";

	  while (collist_here != collist_end) {
	    ColInfoResponse_t cinfo;
	    (*rprotolist_here).p->get_collectinfo (*collist_here, cinfo, err, logout);
	    if (err == noError) {
	      text_t link = "<a href=\"_gwcgi_?a=q&c=" + *collist_here + "\">";
	      if (!cinfo.receptionist.empty()) 
		link = "<a href=\"" + cinfo.receptionist + "?a=q&c=" + *collist_here + "\">";
	      
	      homeextra += "<dt>" + link + *collist_here + "</a></dt>\n";
	      homeextra += "<dd>";
	      if (cinfo.numDocs != 0) homeextra += text_t(cinfo.numDocs) + "_documents_";
	      if (cinfo.numWords != 0) homeextra += text_t(cinfo.numWords) + "_words_";
	      unsigned long last_update = (current_time - cinfo.buildDate) / 86400;
	      homeextra += "_lastupdate_ " + text_t(last_update) + " _ago_</dd>\n";
	    }
	    collist_here ++;
	  }
	  homeextra += "</dl>\n";
	  disp.setmacro ("homeextra", "home", homeextra); 
	}
      }
      rprotolist_here ++;
    }
  }
}

bool pageaction::do_action (cgiargsclass &args, const ColInfoResponse_t &/*collectinfo*/, 
			    recptproto */*collectproto*/, displayclass &disp, 
			    outconvertclass &outconvert, ostream &textout, 
			    ostream &/*logout*/) {

  text_t &arg_p = args["p"];

  textout << outconvert << disp << ("_" + arg_p + ":header_\n")
	  << ("_" + arg_p + ":content_\n")
	  << ("_" + arg_p + ":footer_\n");

  return true;
}
